module Model.Game (Castling, Game, initial, toFEN) where

import Dict
import List
import String

import Model.Board    (..)
import Model.Board    as Board
import Model.Piece    (..)
import Model.Player   (..)
import Model.Position (..)
import Model.Position as Position


type alias Castling = {
    whiteKingside  : Bool,
    whiteQueenside : Bool,
    blackKingside  : Bool,
    blackQueenside : Bool
}

type alias Game = {
    board     : Board,
    player    : Player,
    castling  : Castling,
    enPassant : Maybe Position,
    halfmoves : Int,
    fullmoves : Int
}

initial : Game
initial = { board     = Board.initial
          , player    = White
          , castling  = { whiteKingside  = True
                        , whiteQueenside = True
                        , blackKingside  = True
                        , blackQueenside = True }
          , enPassant = Nothing
          , halfmoves = 0
          , fullmoves = 1 }


-- FEN Notation ----------------------------------------------------------------

toFENPiece : Piece -> String
toFENPiece p =
    case p of
        WhitePawn   -> "P"
        BlackPawn   -> "p"
        WhiteKnight -> "N"
        BlackKnight -> "n"
        WhiteBishop -> "B"
        BlackBishop -> "b"
        WhiteRook   -> "R"
        BlackRook   -> "r"
        WhiteQueen  -> "Q"
        BlackQueen  -> "q"
        WhiteKing   -> "K"
        BlackKing   -> "k"

toFENRank : Board -> Rank -> String
toFENRank b r =
    let pieces       = List.map (\pos -> Dict.get pos b) <| rank r
        folder piece = \(l, c) -> case piece of
            Nothing -> (l, c + 1)
            Just p  -> if c > 0
                then (l ++ toString c ++ toFENPiece p, 0)
                else (l ++ toFENPiece p, 0)
        folded       = List.foldl folder ("", 0) pieces
    in fst folded ++ if snd folded > 0
        then toString <| snd folded
        else ""

toFENBoard : Game -> String
toFENBoard g =
    let toFENRank' = toFENRank g.board
        ranks = [8, 7, 6, 5, 4, 3, 2, 1]
    in String.join "/" <| List.map toFENRank' ranks

toFENPlayer : Game -> String
toFENPlayer g =
    let p = g.player
    in case p of
        White -> "w"
        Black -> "b"

toFENCastling : Game -> String
toFENCastling g =
    let c = g.castling
        s =    (if c.whiteKingside  then "K" else "")
            ++ (if c.whiteQueenside then "Q" else "")
            ++ (if c.blackKingside  then "k" else "")
            ++ (if c.blackQueenside then "q" else "")
    in if s == "" then "-" else s

toFENEnPassant : Game -> String
toFENEnPassant g =
    let e = g.enPassant
    in case e of
        Nothing -> "-"
        Just p  -> Position.show p

toFENHalfmoves : Game -> String
toFENHalfmoves g = toString g.halfmoves

toFENFullmoves : Game -> String
toFENFullmoves g = toString g.fullmoves

toFEN : Game -> String
toFEN g =    toFENBoard g     ++ " "
          ++ toFENPlayer g    ++ " "
          ++ toFENCastling g  ++ " "
          ++ toFENEnPassant g ++ " "
          ++ toFENHalfmoves g ++ " "
          ++ toFENFullmoves g
